package org.jvending.provisioning.impl;

import junit.framework.TestCase;
import org.jvending.registry.RepositoryRegistry;
import org.jvending.registry.hibernate.HibernateDAORegistry;
import org.jvending.provisioning.dao.ClientBundleDAO;
import org.jvending.provisioning.dao.BundleDescriptorDAO;
import org.jvending.provisioning.config.AttributeMatcherRepository;
import org.jvending.provisioning.config.MimeTypeRepository;
import org.jvending.provisioning.config.DeliverableRepository;
import org.jvending.provisioning.stocking.handler.StockingHandlerConfig;
import org.jvending.provisioning.stocking.DataSink;
import org.easymock.MockControl;

import javax.provisioning.BundleRepository;
import javax.provisioning.ProvisioningContext;
import javax.servlet.ServletContext;

import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

public class BundleRepositoryImplTest extends TestCase {

    private BundleRepository bundleRepository;

    private static String baseDir = System.getProperty("basedir");

    public void setUp() {

        RepositoryRegistry repositoryRegistry = RepositoryRegistry.Factory.create();
        HibernateDAORegistry hibernateDaoRegistry = HibernateDAORegistry.Factory.create();
        try {
            repositoryRegistry.loadFromFile(baseDir + "/src/test/resources/registry-config.xml");
        } catch (IOException e) {
            e.printStackTrace();
            fail("Could not load the registry-config file");
        }
            //repositoryRegistry.setRepositoryLoader(new ServletRepositoryLoader(servletContext));
            hibernateDaoRegistry.setRepositoryRegistry(repositoryRegistry);

        ClientBundleDAO clientBundleDAO = (ClientBundleDAO) hibernateDaoRegistry.find("dao:client-bundle");
        assertNotNull("Client bundle DAO is null", clientBundleDAO);

        BundleDescriptorDAO bundleDescriptorDAO =
                (BundleDescriptorDAO) hibernateDaoRegistry.find("dao:bundle-descriptor");
        assertNotNull("Bundle descriptor DAO is null", bundleDescriptorDAO);

        AttributeMatcherRepository matcherRepository = (AttributeMatcherRepository) repositoryRegistry.find("matchers");
        MimeTypeRepository mimeTypeRepository = (MimeTypeRepository) repositoryRegistry.find("mimetype");
        DeliverableRepository deliverableRepository = (DeliverableRepository) repositoryRegistry.find("deliverables");

        MockControl controlServletContext = MockControl.createControl(ServletContext.class);
        ServletContext servletContext = (ServletContext) controlServletContext.getMock();

        MockControl controlProvisioningContext = MockControl.createControl(ProvisioningContext.class);
        ProvisioningContext provisioningContext = (ProvisioningContext) controlProvisioningContext.getMock();

        MockControl controlStockingHandlerConfig = MockControl.createControl(StockingHandlerConfig.class);
        StockingHandlerConfig stockingHandlerConfig = (StockingHandlerConfig) controlStockingHandlerConfig.getMock();

        provisioningContext.getServletContext();
        controlProvisioningContext.setReturnValue(servletContext);

        servletContext.getInitParameter("CONTENT_DELIVERY_URI");
        controlServletContext.setReturnValue(null);

        stockingHandlerConfig.getInitParameter("par-file-output");
        controlStockingHandlerConfig.setReturnValue("./target");

        controlServletContext.replay();
        controlProvisioningContext.replay();
        controlStockingHandlerConfig.replay();

        bundleRepository = new BundleRepositoryImpl(clientBundleDAO, bundleDescriptorDAO, null,
                matcherRepository, mimeTypeRepository, deliverableRepository, provisioningContext);
        ((DataSink) bundleRepository).init(stockingHandlerConfig);

    }

    public void testAddParFile() {

        try {
            FileInputStream fis =
                    new FileInputStream(baseDir + "/src/test/resources/provisioning-archive/provisioning-archive.par");
            bundleRepository.addParFile(fis);
        } catch (FileNotFoundException e) {
            fail("Could not add par file");
        } catch(IOException e)  {
            e.printStackTrace();
            fail("Could not add par file");
        }

    //System.out.println("Need to download the test.par for this test to work.");
    }

}
