/*
 *  JVending - JSP Tag Library
 *  Copyright (C) 2004  Shane Isbell
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.jvending.provisioning.taglib;

import java.io.IOException;

import java.util.Iterator;
import java.util.Set;
import java.util.logging.*;

import javax.provisioning.ProvisioningContext;
import javax.provisioning.Capabilities;

import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.BodyContent;

/**
 * @author Shane Isbell
 * @since 1.3.9a
 */

public class CapabilityNamesTag extends ProvisioningBaseTag {

    private static Logger logger = Logger.getLogger("CapabilityNamesTag");

    private Iterator capabilityNames;

    private Capabilities capabilities;

    private String deviceId;

    public String getDeviceId() {
        return deviceId;
    }

    public void setDeviceId(String deviceId) {
        this.deviceId = deviceId;
    }

    public int doStartTag() throws JspTagException {
        ProvisioningContext provisioningContext = getProvisioningContext();
        //TO DO: not set from parameter
        deviceId = (String) pageContext.getAttribute("deviceId", pageContext.PAGE_SCOPE);

        if(provisioningContext == null || deviceId == null || deviceId.trim() == "") return SKIP_BODY;

        capabilities = provisioningContext.getDeviceTypeCapabilities(deviceId);

        if(capabilities == null) {
            logger.info("Could not find capabilities: Device Id = " + deviceId);
            return SKIP_BODY;
        }

        Set capabilityNameSet = capabilities.getCapabilityNames();
        if(capabilityNameSet == null || capabilityNameSet.size() == 0) return SKIP_BODY;
        capabilityNames = capabilityNameSet.iterator();
        String capabilityName = (String) capabilityNames.next();

        pageContext.setAttribute("capabilityName", capabilityName);
        pageContext.setAttribute("capabilityValues", capabilities.getCapability(capabilityName));
        return EVAL_BODY_BUFFERED;
    }

    protected String getAttribute() {
        return "capabilityName";
    }

    protected Iterator getIterator() {
        return capabilityNames;
    }

    public int doAfterBody() throws JspTagException {     
        BodyContent bodyContent = getBodyContent();
        if(bodyContent != null) {
            buffer.append(bodyContent.getString());
            try {
                bodyContent.clear();
            } catch(IOException e) {
                e.printStackTrace();
            }
        }
       Iterator i = getIterator();
       
       if(i.hasNext()) {
            String capabilityName = (String) i.next();
            pageContext.setAttribute(getAttribute(), capabilityName);
            pageContext.setAttribute("capabilityValues", capabilities.getCapability(capabilityName));
            return EVAL_BODY_BUFFERED;
        } else
            return SKIP_BODY;
    }
}
